@@ -354,7 +354,7 @@ def get_default_class_converter(expected_type):
             return expected_type(**data)
         except TypeError as type_error:
             raise BadRequest(
-                f"invalid parameter in request payload, "
+                "invalid parameter in request payload, "
                 + f"caused by type {_try_get_type_name(expected_type)} or "
                 + "one of its subproperties. Error: "
                 + _generalize_init_type_error_message(type_error)
@@ -410,7 +410,7 @@ class BodyBinder(Binder):
         generic_type = self.get_type_for_generic_iterable(expected_type)
         item_type = self.generic_iterable_annotation_item_type(expected_type)
 
-        if isinstance(item_type, ForwardRef):
+        if isinstance(item_type, ForwardRef):  # pragma: no cover
             from blacksheep.server.normalization import (
                 UnsupportedForwardRefInSignatureError,
             )
@@ -723,7 +723,7 @@ class RouteBinder(SyncBinder):
         expected_type: T = str,
         name: str = None,
         implicit: bool = False,
-        required: bool = False,
+        required: bool = True,
         converter: Optional[Callable] = None,
     ):
         super().__init__(expected_type, name or "route", implicit, required, converter)
