@@ -15,6 +15,7 @@ from typing import (
     Type,
     TypeVar,
     Union,
+    get_type_hints,
 )
 from uuid import UUID
 
@@ -44,6 +45,57 @@ from .bindings import (
 _next_handler_binder = object()
 
 
+# region PEP 563
+
+
+class ParamInfo:
+    __slots__ = ("name", "annotation", "kind", "default", "_str")
+
+    def __init__(self, name, annotation, kind, default, str_repr):
+        self.name = name
+        self.annotation = annotation
+        self.kind = kind
+        self.default = default
+        self._str = str_repr
+
+    def __str__(self) -> str:
+        return self._str
+
+
+def _get_method_annotations_or_throw(method):
+    method_locals = getattr(method, "_locals", None)
+    method_globals = getattr(method, "_globals", None)
+
+    try:
+        return get_type_hints(method, globalns=method_globals, localns=method_locals)
+    except TypeError:
+        if inspect.isclass(method) or hasattr(method, "__call__"):
+            # can be a callable class
+            return get_type_hints(
+                method.__call__, globalns=method_globals, localns=method_locals
+            )
+        raise  # pragma: no cover
+
+
+def _get_method_annotations_base(method):
+    signature = Signature.from_callable(method)
+    params = {
+        key: ParamInfo(
+            value.name, value.annotation, value.kind, value.default, str(value)
+        )
+        for key, value in signature.parameters.items()
+    }
+
+    annotations = _get_method_annotations_or_throw(method)
+    for key, value in params.items():
+        if key in annotations:
+            value.annotation = annotations[key]
+    return params
+
+
+# endregion
+
+
 class NormalizationError(Exception):
     ...
 
@@ -60,7 +112,7 @@ class UnsupportedSignatureError(NormalizationError):
 
 class UnsupportedForwardRefInSignatureError(NormalizationError):
     def __init__(self, unsupported_type):
-        super().__init__(
+        super().__init__(  # pragma: no cover
             f"Cannot normalize method `{unsupported_type}` because its "
             f"signature contains a forward reference (type annotation as string). "
             f"Use type annotations to exact types to fix this error. "
@@ -220,7 +272,7 @@ def _get_parameter_binder(
     # unwrap the Optional[] annotation, if present:
     is_root_optional, annotation = _check_union(parameter, original_annotation, method)
 
-    if isinstance(annotation, (str, ForwardRef)):
+    if isinstance(annotation, (str, ForwardRef)):  # pragma: no cover
         raise UnsupportedForwardRefInSignatureError(original_annotation)
 
     # 1. is the type annotation of BoundValue[T] type?
@@ -230,7 +282,7 @@ def _get_parameter_binder(
 
         is_optional, expected_type = _check_union(parameter, expected_type, method)
 
-        if isinstance(expected_type, (str, ForwardRef)):
+        if isinstance(expected_type, (str, ForwardRef)):  # pragma: no cover
             raise UnsupportedForwardRefInSignatureError(expected_type)
 
         parameter_name = annotation.name or name
@@ -292,8 +344,7 @@ def get_parameter_binder(
 def _get_binders_for_function(
     method: Callable[..., Any], services: Services, route: Optional[Route]
 ) -> List[Binder]:
-    signature = Signature.from_callable(method)
-    parameters = signature.parameters
+    parameters = _get_method_annotations_base(method)
     body_binder = None
 
     binders = []
@@ -375,7 +426,7 @@ def get_sync_wrapper(
     services: Services,
     route: Route,
     method: Callable[..., Any],
-    params: Mapping[str, inspect.Parameter],
+    params: Mapping[str, ParamInfo],
     params_len: int,
 ) -> Callable[[Request], Awaitable[Response]]:
     if params_len == 0:
@@ -411,7 +462,7 @@ def get_async_wrapper(
     services: Services,
     route: Route,
     method: Callable[..., Any],
-    params: Mapping[str, inspect.Parameter],
+    params: Mapping[str, ParamInfo],
     params_len: int,
 ) -> Callable[[Request], Awaitable[Response]]:
     if params_len == 0:
@@ -468,7 +519,7 @@ def normalize_handler(
     method = route.handler
 
     sig = Signature.from_callable(method)
-    params = sig.parameters
+    params = _get_method_annotations_base(method)
     params_len = len(params)
 
     if any(
@@ -547,8 +598,7 @@ def normalize_middleware(
     ):
         raise ValueError("Middlewares must be asynchronous functions")
 
-    sig = Signature.from_callable(middleware)
-    params = sig.parameters
+    params = _get_method_annotations_base(middleware)
 
     if _is_basic_middleware_signature(params):
         return middleware
