@@ -1,13 +1,12 @@
-from openapidocs.common import Format
-from blacksheep.server.openapi.exceptions import (
-    DuplicatedContentTypeDocsException,
-    UnsupportedUnionTypeException,
+from blacksheep.server.openapi.docstrings import (
+    DocstringInfo,
+    get_handler_docstring_info,
 )
 import inspect
 from dataclasses import fields, is_dataclass
 from datetime import date, datetime
 from enum import Enum, IntEnum
-from typing import Any, Dict, List, Optional, Tuple, Type, Union, ForwardRef
+from typing import Any, Dict, ForwardRef, List, Mapping, Optional, Tuple, Type, Union
 from uuid import UUID
 
 from blacksheep.server.bindings import (
@@ -19,12 +18,17 @@ from blacksheep.server.bindings import (
     RouteBinder,
     empty,
 )
+from blacksheep.server.openapi.exceptions import (
+    DuplicatedContentTypeDocsException,
+    UnsupportedUnionTypeException,
+)
 from blacksheep.server.routing import Router
+from openapidocs.common import Format
 from openapidocs.v3 import (
     Components,
     Example,
-    Info,
     Header,
+    Info,
     MediaType,
     OpenAPI,
     Operation,
@@ -33,16 +37,19 @@ from openapidocs.v3 import (
     PathItem,
     Reference,
     RequestBody,
-    Server,
 )
 from openapidocs.v3 import Response as ResponseDoc
-from openapidocs.v3 import Schema, ValueFormat, ValueType
+from openapidocs.v3 import Schema, Server, ValueFormat, ValueType
 
 from ..application import Application
 from .common import (
     APIDocsHandler,
-    HeaderInfo,
     ContentInfo,
+    EndpointDocs,
+    HeaderInfo,
+    ParameterInfo,
+    ParameterSource,
+    RequestBodyInfo,
     ResponseExample,
     ResponseInfo,
     ResponseStatusType,
@@ -69,6 +76,23 @@ def check_union(object_type: Any) -> Tuple[bool, Any]:
     return False, object_type
 
 
+def is_ignored_parameter(param_name: str, matching_binder: Optional[Binder]) -> bool:
+    # if a binder is used, handle only those that can be mapped to a type of
+    # OpenAPI Documentation parameter's location
+    if matching_binder and not isinstance(
+        matching_binder,
+        (
+            QueryBinder,
+            RouteBinder,
+            HeaderBinder,
+            CookieBinder,
+        ),
+    ):
+        return True
+
+    return param_name == "request" or param_name == "services"
+
+
 class OpenAPIHandler(APIDocsHandler[OpenAPI]):
     """
     Handles the automatic generation of OpenAPI Documentation, specification v3
@@ -182,6 +206,7 @@ class OpenAPIHandler(APIDocsHandler[OpenAPI]):
 
             if isinstance(child_type, str):
                 # this is a forward reference
+                child_type = child_type.strip("'")
                 properties[field.name] = Reference(f"#/components/schemas/{child_type}")
             else:
                 properties[field.name] = self.get_schema_by_type(child_type)
@@ -328,14 +353,22 @@ class OpenAPIHandler(APIDocsHandler[OpenAPI]):
             return Schema(type=ValueType.STRING, enum=[v.value for v in object_type])
         return None
 
+    def _get_body_binder(self, handler: Any) -> Optional[BodyBinder]:
+        return next(
+            (binder for binder in handler.binders if isinstance(binder, BodyBinder)),
+            None,
+        )
+
+    def _get_binder_by_name(self, handler: Any, name: str) -> Optional[Binder]:
+        return next(
+            (binder for binder in handler.binders if binder.parameter_name == name),
+            None,
+        )
+
     def get_request_body(self, handler: Any) -> Union[None, RequestBody, Reference]:
         if not hasattr(handler, "binders"):
             return None
-        binders: List[Binder] = handler.binders
-
-        body_binder = next(
-            (binder for binder in binders if isinstance(binder, BodyBinder)), None
-        )
+        body_binder = self._get_body_binder(handler)
 
         if body_binder is None:
             return None
@@ -373,13 +406,21 @@ class OpenAPIHandler(APIDocsHandler[OpenAPI]):
             return ParameterLocation.HEADER
         return None
 
+    def _parameter_source_to_openapi_obj(
+        self, value: ParameterSource
+    ) -> ParameterLocation:
+        return ParameterLocation[value.value.upper()]
+
     def get_parameters(
         self, handler: Any
     ) -> Optional[List[Union[Parameter, Reference]]]:
         if not hasattr(handler, "binders"):
             return None
         binders: List[Binder] = handler.binders
-        parameters: List[Union[Parameter, Reference]] = []
+        parameters: Mapping[str, Union[Parameter, Reference]] = {}
+
+        docs = self.get_handler_docs(handler)
+        parameters_info = (docs.parameters if docs else None) or dict()
 
         for binder in binders:
             location = self.get_parameter_location_for_binder(binder)
@@ -394,17 +435,34 @@ class OpenAPIHandler(APIDocsHandler[OpenAPI]):
             else:
                 required = binder.required and binder.default is empty
 
-            parameters.append(
-                Parameter(
-                    name=binder.parameter_name,
-                    in_=location,
-                    required=required or None,
-                    schema=self.get_schema_by_type(binder.expected_type),
-                    description="",
-                )
+            # did the user specified information about the parameter?
+            param_info = parameters_info.get(binder.parameter_name)
+
+            parameters[binder.parameter_name] = Parameter(
+                name=binder.parameter_name,
+                in_=location,
+                required=required or None,
+                schema=self.get_schema_by_type(binder.expected_type),
+                description=param_info.description if param_info else "",
+                example=param_info.example if param_info else None,
             )
 
-        return parameters
+        for key, param_info in parameters_info.items():
+            if key not in parameters:
+                parameters[key] = Parameter(
+                    name=key,
+                    in_=self._parameter_source_to_openapi_obj(
+                        param_info.source or ParameterSource.QUERY
+                    ),
+                    required=param_info.required,
+                    schema=self.get_schema_by_type(param_info.value_type)
+                    if param_info.value_type
+                    else None,
+                    description=param_info.description,
+                    example=param_info.example,
+                )
+
+        return list(parameters.values())
 
     def _get_media_type_from_content_doc(self, content_doc: ContentInfo) -> MediaType:
         media_type = MediaType()
@@ -516,6 +574,67 @@ class OpenAPIHandler(APIDocsHandler[OpenAPI]):
     def on_docs_generated(self, docs: OpenAPI) -> None:
         docs.servers = self.servers
 
+    def _merge_documentation(
+        self,
+        handler,
+        endpoint_docs: EndpointDocs,
+        docstring_info: DocstringInfo,
+    ) -> None:
+        if not endpoint_docs.description and docstring_info.description:
+            endpoint_docs.description = docstring_info.description
+
+        if not endpoint_docs.summary and docstring_info.summary:
+            endpoint_docs.summary = docstring_info.summary
+
+        for param_name, param_info in docstring_info.parameters.items():
+            if endpoint_docs.parameters is None:
+                endpoint_docs.parameters = {}
+
+            # did the user specify parameter information explicitly, using @docs?
+            matching_parameter = endpoint_docs.parameters.get(param_name)
+
+            if matching_parameter is None:
+                matching_binder = self._get_binder_by_name(handler, param_name)
+
+                if isinstance(matching_binder, BodyBinder):
+                    if endpoint_docs.request_body is None:
+                        endpoint_docs.request_body = RequestBodyInfo(
+                            description=param_info.description
+                        )
+                    elif not endpoint_docs.request_body.description:
+                        endpoint_docs.request_body.description = param_info.description
+                    continue
+
+                if is_ignored_parameter(param_name, matching_binder):
+                    # this must not be documented in OpenAPI Documentation!
+                    continue
+
+                assert isinstance(endpoint_docs.parameters, dict)
+                endpoint_docs.parameters[param_name] = ParameterInfo(
+                    value_type=param_info.value_type,
+                    required=param_info.required,
+                    description=param_info.description,
+                    source=param_info.source or ParameterSource.QUERY,
+                )
+            else:
+                matching_parameter.description = param_info.description
+
+                if (
+                    matching_parameter.value_type is None
+                    and param_info.value_type is not None
+                ):
+                    matching_parameter.value_type = param_info.value_type
+
+    def _apply_docstring(self, handler, docs: Optional[EndpointDocs]) -> None:
+        if not self.use_docstrings:  # pragma: no cover
+            return
+        docstring_info = get_handler_docstring_info(handler)
+
+        if docstring_info is not None:
+            if docs is None:
+                docs = self.get_handler_docs_or_set(handler)
+            self._merge_documentation(handler, docs, docstring_info)
+
     def get_routes_docs(self, router: Router) -> Dict[str, PathItem]:
         """Obtains a documentation object from the routes defined in a router."""
         paths_doc: Dict[str, PathItem] = {}
@@ -527,6 +646,7 @@ class OpenAPIHandler(APIDocsHandler[OpenAPI]):
             for method, route in conf.items():
                 handler = self._get_request_handler(route)
                 docs = self.get_handler_docs(handler)
+                self._apply_docstring(handler, docs)
 
                 operation = Operation(
                     description=self.get_description(handler),
